/*******************************************************************************
 *
 * File Name: LDreferenceClass.h
 * Author: Richard Baxter
 * Project: Generic Ldraw Construct Functions
 * Project Version: 1z3b 19-Sept-09 (COMPILE_OLC only release)
 *
 *******************************************************************************/



/*LDreferenceClass.cpp: Defines a class for references to submodels/parts/units within a scene file*/

#ifndef HEADER_LD_REFERENCE_CLASS
#define HEADER_LD_REFERENCE_CLASS


#include "SHAREDglobalDefs.h"
#include "SHAREDvars.h"


	#include <stdio.h>
	#include <stdlib.h>
	#include <string>
	#include <iostream>
	#include <fstream>
	#include <time.h>
	#include <math.h>
	using namespace std;


#define TAL_FILE_COLOUR_BLACK_RGB "000000"
#define TAL_FILE_COLOUR_BLUE_RGB "0000F0"
#define TAL_FILE_COLOUR_GREEN_RGB "00C000"
#define TAL_FILE_COLOUR_CYAN_RGB "00F0F0"
#define TAL_FILE_COLOUR_RED_RGB "F00000"
#define TAL_FILE_COLOUR_MAGENTA_RGB "FF00FF"
#define TAL_FILE_COLOUR_BROWN_RGB "C06000"
#define TAL_FILE_COLOUR_LIGHTGREY_RGB "C0C0C0"
#define TAL_FILE_COLOUR_DARKGREY_RGB "606060"
#define TAL_FILE_COLOUR_YELLOW_RGB "FFFF00"
#define TAL_FILE_COLOUR_WHITE_RGB "FFFFFF"
#define TAL_FILE_COLOUR_ORANGE_RGB "FF0600"
#define TAL_FILE_COLOUR_PURPLE_RGB "A000FF"

#define DAT_FILE_COLOUR_BLACK_RGB_R (0)
#define DAT_FILE_COLOUR_BLUE_RGB_R (0)
#define DAT_FILE_COLOUR_GREEN_RGB_R (0)
#define DAT_FILE_COLOUR_CYAN_RGB_R (0)
#define DAT_FILE_COLOUR_RED_RGB_R (240)
#define DAT_FILE_COLOUR_MAGENTA_RGB_R (255)
#define DAT_FILE_COLOUR_BROWN_RGB_R (192)
#define DAT_FILE_COLOUR_LIGHTGREY_RGB_R (192)
#define DAT_FILE_COLOUR_DARKGREY_RGB_R (96)
#define DAT_FILE_COLOUR_YELLOW_RGB_R (255)
#define DAT_FILE_COLOUR_WHITE_RGB_R (255)
#define DAT_FILE_COLOUR_ORANGE_RGB_R (255)
#define DAT_FILE_COLOUR_PURPLE_RGB_R (160)

#define DAT_FILE_COLOUR_BLACK_RGB_G (0)
#define DAT_FILE_COLOUR_BLUE_RGB_G (0)
#define DAT_FILE_COLOUR_GREEN_RGB_G (192)
#define DAT_FILE_COLOUR_CYAN_RGB_G (240)
#define DAT_FILE_COLOUR_RED_RGB_G (0)
#define DAT_FILE_COLOUR_MAGENTA_RGB_G (0)
#define DAT_FILE_COLOUR_BROWN_RGB_G (96)
#define DAT_FILE_COLOUR_LIGHTGREY_RGB_G (192)
#define DAT_FILE_COLOUR_DARKGREY_RGB_G (96)
#define DAT_FILE_COLOUR_YELLOW_RGB_G (255)
#define DAT_FILE_COLOUR_WHITE_RGB_G (255)
#define DAT_FILE_COLOUR_ORANGE_RGB_G (96)
#define DAT_FILE_COLOUR_PURPLE_RGB_G (0)

#define DAT_FILE_COLOUR_BLACK_RGB_B (0)
#define DAT_FILE_COLOUR_BLUE_RGB_B (240)
#define DAT_FILE_COLOUR_GREEN_RGB_B (0)
#define DAT_FILE_COLOUR_CYAN_RGB_B (240)
#define DAT_FILE_COLOUR_RED_RGB_B (0)
#define DAT_FILE_COLOUR_MAGENTA_RGB_B (255)
#define DAT_FILE_COLOUR_BROWN_RGB_B (0)
#define DAT_FILE_COLOUR_LIGHTGREY_RGB_B (192)
#define DAT_FILE_COLOUR_DARKGREY_RGB_B (96)
#define DAT_FILE_COLOUR_YELLOW_RGB_B (0)
#define DAT_FILE_COLOUR_WHITE_RGB_B (255)
#define DAT_FILE_COLOUR_ORANGE_RGB_B (0)
#define DAT_FILE_COLOUR_PURPLE_RGB_B (255)



#define DAT_FILE_COLOUR_BLACK (0)
#define DAT_FILE_COLOUR_BLUE (1)
#define DAT_FILE_COLOUR_GREEN (2)
#define DAT_FILE_COLOUR_CYAN (3)
#define DAT_FILE_COLOUR_RED (4)
#define DAT_FILE_COLOUR_MAGENTA (5)
#define DAT_FILE_COLOUR_BROWN (6)
#define DAT_FILE_COLOUR_LIGHTGREY (7)
#define DAT_FILE_COLOUR_DARKGREY (8)
#define DAT_FILE_COLOUR_YELLOW (14)
#define DAT_FILE_COLOUR_WHITE (15)
#define DAT_FILE_COLOUR_ORANGE (25)
#define DAT_FILE_COLOUR_PURPLE (22)
#define DAT_FILE_COLOUR_TRANPARENT_MOD (32)
#define DAT_FILE_DEFAULT_COLOUR (16)
#define DAT_FILE_DEFAULT_COLOUR_EDGELINE (24)

#define DAT_FILE_FIRST_RGB_COLOUR (0xFF)	//255 - NB RGB colours are expressed as FFredGreenBlue; eg Red = FFFF0000 [NB no transparency is supported - as 8 bit padding + 24 bit colour = 32bit]

#define DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX (128)
#define DAT_FILE_REF_NUM_OF_MATRIX_VALUES (9)
#define DAT_FILE_REF_NUM_OF_COLOUR_VALUES (1)
#define DAT_FILE_REF_NUM_OF_TYPE_VALUES (1)
#define DAT_FILE_REF_NUM_OF_POSITION_VALUES (3)
#define DAT_FILE_REF_NUM_OF_SUBMODEL_VALUES (1)
#define DAT_FILE_REF_NUM_DATA_VALUES (DAT_FILE_REF_NUM_OF_TYPE_VALUES + DAT_FILE_REF_NUM_OF_COLOUR_VALUES + DAT_FILE_REF_NUM_OF_POSITION_VALUES + DAT_FILE_REF_NUM_OF_MATRIX_VALUES + DAT_FILE_REF_NUM_OF_SUBMODEL_VALUES)
#define DAT_FILE_DATA_VALUE_MAX_LENGTH (32)	//OLD;15 			//assumes matrix / position data - fraction values have a max size of 10 characters
#define DAT_FILE_DATA_VALUE_RECOMMENDED_LENGTH (6) 				//[negative sign] + [1 value before DP] + [DP] + [3 values after DP]

#define DAT_FILE_REF_MAX_SIZE (DAT_FILE_REF_NUM_OF_TYPE_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_REF_NUM_OF_COLOUR_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_POSITION_VALUES + DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_MATRIX_VALUES + DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX + DAT_FILE_REF_NUM_DATA_VALUES)
					/*formatted;
					#define DAT_FILE_REF_MAX_SIZE
						(DAT_FILE_REF_NUM_OF_TYPE_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH +
						DAT_FILE_REF_NUM_OF_COLOUR_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH +
						DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_POSITION_VALUES +
						DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_MATRIX_VALUES +
						DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX +
						DAT_FILE_REF_NUM_DATA_VALUES)
					*/
									/*NB there is 2 (integer) + 3 (fractional) + 9 (fractional) + 1 + 15 (spaces) = a total of 15 values are present*/
#define DAT_FILE_REF_MATRIX_MAX_LENGTH  (DAT_FILE_DATA_VALUE_MAX_LENGTH*DAT_FILE_REF_NUM_OF_MATRIX_VALUES + DAT_FILE_REF_NUM_OF_MATRIX_VALUES)
#define DAT_FILE_REF_POS_COORD_MAX_LENGTH (DAT_FILE_DATA_VALUE_MAX_LENGTH*DAT_FILE_REF_NUM_OF_POSITION_VALUES + DAT_FILE_REF_NUM_OF_POSITION_VALUES)
#ifdef USE_LRRC
	#define DAT_FILE_MAX_NUM_OF_REFERENCES (10000U)	//10 thousand references
#elif defined USE_ANN
	#define DAT_FILE_MAX_NUM_OF_REFERENCES (1000000U)	//1 million references
#else
	#define DAT_FILE_MAX_NUM_OF_REFERENCES (100000U)	//100 thousand references
#endif

	/*new parameters added to parseFile() 18-mar-07*/
#define REFERENCE_TYPE_COMMENT (0)
#define REFERENCE_TYPE_SUBMODEL (1)
#define REFERENCE_TYPE_LINE (2)
#define REFERENCE_TYPE_TRI (3)
#define REFERENCE_TYPE_QUAD (4)
#define REFERENCE_TYPE_OPTIONALLINE (5)
#define REFERENCE_TYPE_UNDEFINED (9)
#define PRIM_REF_MAX_NUM_OF_POSITION_VALUES (12)
#define PRIMATIVE_REF_MAX_SIZE (DAT_FILE_REF_NUM_OF_TYPE_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_REF_NUM_OF_COLOUR_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_DATA_VALUE_MAX_LENGTH * PRIM_REF_MAX_NUM_OF_POSITION_VALUES)



	/*a matrix is defined by 3 vectors side by side*/

class Reference{
private:
	/*There are currently no private attributes of this class*/
public:

	Reference(void); // constructor declaration
	Reference(string referenceName, int referenceColour, bool createNewSubmodel); // constructor declaration
	Reference(bool createNewSubmodel);
	~Reference();	//  and destructor.

	vec relativePosition;
	vec absolutePosition;
	mat deformationMatrix;
	mat absoluteDeformationMatrix;
	unsigned int colour;
	string name;
	Reference * next;

		/*new parameters added to parseFile() 18-mar-07*/
	vec vertex1relativePosition;
	vec vertex2relativePosition;
	vec vertex3relativePosition;
	vec vertex4relativePosition;
	vec vertex1absolutePosition;
	vec vertex2absolutePosition;
	vec vertex3absolutePosition;
	vec vertex4absolutePosition;
	
	vec vertex1absolutePositionBackup;
	vec vertex2absolutePositionBackup;
	vec vertex3absolutePositionBackup;
	vec vertex4absolutePositionBackup;
	bool referenceEnabledMethod2DOD;
	
	int type;

#ifdef USE_RT
	unsigned int absoluteColour;
#endif
	
	/*Additional values used with recursive parser*/
	bool isSubModelReference;
	Reference * firstReferenceWithinSubModel;
};

void copyReferencePosition(Reference * r, Reference * referenceToCopy);

int obtainReferencePlayerID(Reference * referenceInThisPhaseStartSceneFile);

void convertLdrawColourToDatFileRGB(int dataFileColour, colour * col);


#endif

